@@ -46,23 +46,37 @@ class RemoteScheduler(Scheduler):
     def _wait(self):
         time.sleep(30)
 
-    def _request(self, url, data, log_exceptions=True, attempts=3):
-        # TODO(erikbern): do POST requests instead
-        data = {'data': json.dumps(data)}
+    def _get(self, url, data):
         url = 'http://%s:%d%s?%s' % \
-            (self._host, self._port, url, urllib.urlencode(data))
+              (self._host, self._port, url, urllib.urlencode(data))
+        return urllib2.Request(url)
 
-        req = urllib2.Request(url)
+    def _post(self, url, data):
+        url = 'http://%s:%d%s' % (self._host, self._port, url)
+        return urllib2.Request(url, urllib.urlencode(data))
+
+    def _request(self, url, data, log_exceptions=True, attempts=3):
+        data = {'data': json.dumps(data)}
+
+        req = self._post(url, data)
         last_exception = None
-        for attempt in xrange(attempts):
+        attempt = 0
+        while attempt < attempts:
+            attempt += 1
             if last_exception:
                 logger.info("Retrying...")
                 self._wait()  # wait for a bit and retry
             try:
                 response = urllib2.urlopen(req, None, self._connect_timeout)
                 break
-            except urllib2.URLError, last_exception:
-                if log_exceptions:
+            except urllib2.URLError as last_exception:
+                if isinstance(last_exception, urllib2.HTTPError) and last_exception.code == 405:
+                    # TODO(f355): 2014-08-29 Remove this fallback after several weeks
+                    logger.warning("POST requests are unsupported. Please upgrade scheduler ASAP. Falling back to GET for now.")
+                    req = self._get(url, data)
+                    last_exception = None
+                    attempt -= 1
+                elif log_exceptions:
                     logger.exception("Failed connecting to remote scheduler %r", self._host)
                 continue
         else:
