@@ -30,6 +30,7 @@ import signal
 from rpc import RemoteSchedulerResponder
 import task_history
 import logging
+import warnings
 logger = logging.getLogger("luigi.server")
 
 
@@ -63,6 +64,8 @@ class RPCHandler(tornado.web.RequestHandler):
         else:
             self.send_error(404)
 
+    post = get
+
 
 class BaseTaskHistoryHandler(tornado.web.RequestHandler):
     def initialize(self, api):
